
/* Step 1 - Worktable */
USE AdventureWorks
GO
CREATE TABLE SalesOrderTotalsMonthly
(
CustomerID int NOT NULL,
OrderMonth int NOT NULL,
SubTotal money NOT NULL
)
GO
INSERT SalesOrderTotalsMonthly
SELECT CustomerID, DATEPART(m, OrderDate), SubTotal
FROM Sales.SalesOrderHeader
WHERE CustomerID IN (1,2,4,6)
GO

/* Step 2 - sales for each month are output by customer */

SELECT * FROM SalesOrderTotalsMonthly
PIVOT (SUM(SubTotal) FOR OrderMonth IN
([1], [2], [3], [4], [5], [6], [7], [8], [9], [10], [11], [12])) AS a
GO

/* Step 3 - Worktable */

CREATE TABLE SalesOrderTotalsYearly
(
CustomerID int NOT NULL,
OrderYear int NOT NULL,
SubTotal money NOT NULL
)
GO
INSERT SalesOrderTotalsYearly
SELECT CustomerID, YEAR(OrderDate), SubTotal
FROM Sales.SalesOrderHeader
WHERE CustomerID IN (1,2,4,6,35)
GO

/* Step 4 - each year's sales are output by customer */

SELECT * FROM SalesOrderTotalsYearly
PIVOT (SUM(SubTotal) FOR OrderYear IN ([2002], [2003], [2004])) AS a
GO
SELECT * FROM SalesOrderTotalsYearly
PIVOT (SUM(SubTotal) FOR CustomerID in ([1], [2], [4], [6])) AS a
GO

/* Step 5 - create a worktable and pivot the table so that each years sales are output by customer */

CREATE TABLE YearlySalesPivot
(
OrderYear int NOT NULL,
[1] money NULL,
[2] money NULL,
[4] money NULL,
[6] money NULL
)
GO
INSERT YearlySalesPivot
SELECT * FROM SalesOrderTotalsYearly
PIVOT (SUM(SubTotal) FOR CustomerID IN ([1], [2], [4], [6])) AS a
GO

/* Step 6 - UnPivot */
SELECT * FROM YearlySalesPivot
UNPIVOT (SubTotal FOR CustomerID IN ([1], [2], [4], [6]))
a
ORDER BY CustomerID
-- Displays unpivoted results
SELECT * FROM YearlySalesPivot
-- Displays pivoted results stored in the worktable
GO

/* Cleanup */
DROP TABLE YearlySalesPivot
DROP TABLE SalesOrderTotalsYearly
DROP TABLE SalesOrderTotalsMonthly
GO
